/*
    Part of X2GoAdminCenter.
    Copyright (C) 2011-2015 Oleksandr Shneyder<oleksandr.shneyder@obviously-nice.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "x2gogroupadminwindow.h"
#include <QTimer>
#include <QPixmap>
#include <QInputDialog>
#include <QDebug>
#include <QMessageBox>
#include <centerinterface.h>

void X2GoGroupAdminWindow::installTranslator()
{
    QTranslator* trans=new QTranslator();
    QString filename=QString ( ":/x2gogroupadmingui_%1" ).arg (
                         QLocale::system().name() );
    filename=filename.toLower();
    if ( !trans->load ( filename ) )
    {
        qDebug ( "Can't load translator (%s) !\n",
                 filename.toLocal8Bit().data() );
    }
    else
    {
        QCoreApplication::installTranslator ( trans );
    }


    QTranslator* qtTranslator=new QTranslator;
    filename=QString ( ":/qt_%1" ).arg ( QLocale::system().name() );
    if ( !qtTranslator->load ( filename ) )
    {
        qDebug()<< "Can't load translator "<<
        filename.toLocal8Bit().data() ;
    }
    else
    {
        QCoreApplication::installTranslator ( qtTranslator );
    }
}


X2GoGroupAdminWindow::X2GoGroupAdminWindow()
{
    installTranslator();
    NAME =tr("X2Go group administration");
    DESCRIPTION= tr("Use this module to administrate groups");

    DEVELOPERS= "Oleksandr Shneyder &lt;oleksandr.shneyder@obviously-nice.de&gt;<br>"
                "Heinz-Markus Graesing &lt;heinz-m.graesing@obviously-nice.de&gt;";

    TRANSLATORS ="Heinz-Markus Graesing &lt;heinz-m.graesing@obviously-nice.de&gt;";

    ABOUT= "<b>"+NAME + "-" + VERSION + "</b><br><br>"
           +DESCRIPTION+"<br><br><b>"
           + tr("Developers:")+"</b><br>" + DEVELOPERS+ "<br><br><b>"
           + tr("Translators:")+"</b><br>"+ TRANSLATORS;



    qDebug()<<"group admin constructor";
    hidden=true;
    setupUi ( this );
    gbGroup->setEnabled(false);
    gbAllGroups->setEnabled(false);
    centerInstance=0l;
    pbAddGroup->setIcon(QPixmap(":/png/user-group-new.png"));
    pbDelGroup->setIcon(QPixmap(":/png/user-group-delete.png"));
    treeAllUsers->headerItem()->setText(0,"");
}

X2GoGroupAdminWindow::~X2GoGroupAdminWindow()
{

}

void X2GoGroupAdminWindow::startPlugin()
{
    lNameLabel->hide();
    eName->hide();
    sbGid->setValue(0);
    eDesc->setText(QString::null);
    cbType->clear();
    listGroupUsers->clear();
    treeAllUsers->clear();
    treeGroups->clear();
    gbGroup->setEnabled(false);
    gbAllGroups->setEnabled(false);

    changed=false;
    adminMode=true;

    QList<int> sizes;
    sizes<<(gbAllGroups->width()+gbGroup->width())/2<<sizes[0];
    splitter->setSizes(sizes);
    changed=false;
    hidden=false;
    pbApply->setEnabled(false);
    closing=false;
    lName->setText("");
    rpc= new MaiaXmlRpcClient(QUrl(centerInstance->getUrl()),this);

    QSslConfiguration config=rpc->sslConfiguration();
    config.setProtocol(QSsl::AnyProtocol);
    rpc->setSslConfiguration(config);

    connect(rpc, SIGNAL(sslErrors(QNetworkReply*,QList<QSslError>)), this,
            SLOT(slotHandleSslErrors(QNetworkReply*,QList<QSslError>)));
    getGroups();

}


void X2GoGroupAdminWindow::slotApply()
{
    //add support for other atributes
    QVariantList args;
    QStringList newMembers, obsoleteMembers;

    if (currentGroup.newGroup)
    {
        QString name=eName->text();

        if ( (!groupAttributes[GNAME].unicode && !stringIsAscii(name) )|| name.length()<=0)
        {
            QMessageBox::critical(this,tr("Error"),tr("Invalid input: \"Name\""));
            return;
        }
        if (!isAttributeUnic(COL_GNAME,name))
        {
            QMessageBox::critical(this,tr("Error"),tr("Group already exists: ")+name);
            return;
        }
        currentGroup.name=name;
    }
    args<<centerInstance->getLogin()<<centerInstance->getPassword()<<currentGroup.name;
    if (groupAttributes[TYPE].supported)
    {
        currentGroup.type=cbType->currentText();
        args<<"type:"+currentGroup.type;
    }
    if (currentGroup.password!=currentGroup.newpassword)
    {
        args<<"password:"+currentGroup.newpassword;
    }
    QStringList actUsers;
    for (int i=0;i<listGroupUsers->count();++i)
    {
        actUsers<<listGroupUsers->item(i)->text();
        if (!currentGroup.users.contains(listGroupUsers->item(i)->text()))
            newMembers<<listGroupUsers->item(i)->text();
    }
    if (newMembers.count())
        args<<"newmembers:"+newMembers.join(";");

    for (int i=0; i<currentGroup.users.count();++i)
    {
        if (!actUsers.contains(currentGroup.users[i]))
            obsoleteMembers<<currentGroup.users[i];
    }
    if (obsoleteMembers.count())
        args<<"obsoletemembers:"+obsoleteMembers.join(";");

    if (groupAttributes[DESC].supported)
    {
        QString desc=eDesc->text();
        if ( (!groupAttributes[DESC].unicode && !stringIsAscii(desc) )|| desc.length()<=0)
        {
            QMessageBox::critical(this,tr("Error"),tr("Invalid input: \"Description\""));
            return;
        }

        if (currentGroup.desc!=desc)
            args<<"description:"+desc;
    }
    if (groupAttributes[GID].supported)
    {
        QString gid=QString::number(sbGid->value());

        if (currentGroup.gid!=gid)
        {
            if (!isAttributeUnic(COL_GID,gid))
            {
                QMessageBox::critical(this,tr("Error"),tr("Group with this group ID already exists: ")+gid);
                return;
            }
            args<<"gid:"+gid;
        }

    }


    if (args.count()>3)
    {
        if (currentGroup.newGroup)
            rpc->call("x2goadmin.addGroup",args, this, SLOT(slotGroupModified(QVariant&)),
                      this, SLOT(slotRpcFault(int, const QString&)));
        else
            rpc->call("x2goadmin.modifyGroup",args, this, SLOT(slotGroupModified(QVariant&)),
                      this, SLOT(slotRpcFault(int, const QString&)));
    }
    else
    {
        QVariant result;
        result="OK";
        slotGroupModified(result);
    }
    gbAllGroups->setEnabled(false);
    gbGroup->setEnabled(false);
    centerInstance->setStatus(tr("Retrieving information from server, please wait..."));
}

void X2GoGroupAdminWindow::slotGroupModified(QVariant& result)
{
    //when ready  check if closing, set closing to false and close plugin
    if (hidden)
        return;
    centerInstance->setStatus(tr("Ready"));

    gbAllGroups->setEnabled(false);
    gbGroup->setEnabled(false);
    QString res=result.toString();
    qDebug()<<res;
    if (!checkResult(res))
    {
        centerInstance->backToList();
        return;
    }
    if (closing)
    {
        closing=false;
        changed=false;
        centerInstance->backToList();
        return;
    }
    changed=false;
    pbApply->setEnabled(false);
    startPlugin();;
}

void X2GoGroupAdminWindow::getGroups()
{
    QVariantList args;
    args<<centerInstance->getLogin()<<centerInstance->getPassword();
    gbAllGroups->setEnabled(false);
    treeAllUsers->clear();
    treeGroups->clear();
    rpc->call("x2goadmin.getGroups",args, this, SLOT(slotGroupList(QVariant&)),
              this, SLOT(slotRpcFault(int, const QString&)));
    centerInstance->setStatus(tr("Retrieving information from server, please wait..."));
}




void X2GoGroupAdminWindow::slotGroupList(QVariant& result)
{
    if (hidden)
        return;
    QString res=result.toString();
    if (!checkResult(res))
    {
        centerInstance->backToList();
        return;
    }
//     qDebug()<<res;
    int firstAttrPos=res.indexOf("BEGIN_GROUPATTR");
    int lastAttrPos=res.indexOf("END_GROUPATTR");
    QString attrs=res.mid(firstAttrPos, lastAttrPos-firstAttrPos);
    attrs.replace("BEGIN_GROUPATTR\n","");
    initGroupAttr(attrs);
    initGroupList(res);
    centerInstance->setStatus(tr("Ready"));
}

void X2GoGroupAdminWindow::initGroupAttr(const QString& attrString)
{
    for (int i=0; i< GROUPATTR;++i)
    {
        groupAttributes[i].editable=false;
        groupAttributes[i].supported=false;
        groupAttributes[i].unicode=false;
        groupAttributes[i].oncreate=true;
    }
    QStringList attrList=attrString.split("\n",QString::SkipEmptyParts);
    QString attrLine;
    foreach (attrLine, attrList)
    {
        QStringList attrWords=attrLine.split(":",QString::SkipEmptyParts);
        int index=0;
        if (attrWords[0]=="name")
            index=GNAME;
        if (attrWords[0]=="gid")
            index=GID;
        if (attrWords[0]=="password")
            index=PASSWORD;
        if (attrWords[0]=="description")
            index=DESC;
        if (attrWords[0]=="type")
            index=TYPE;
        if (attrWords[0]=="members")
            index=MEMBERS;

        groupAttributes[index].supported=true;

        QStringList attrs=attrWords[1].split(";");
        QString uattr;
        foreach(uattr, attrs)
        {
            QStringList nameval=uattr.split("=");
            if (nameval[0]=="edit")
                groupAttributes[index].editable=(nameval[1]!="0");
            if (nameval[0]=="unicode")
                groupAttributes[index].unicode=(nameval[1]!="0");
            if (nameval[0]=="oncreate")
                groupAttributes[index].oncreate=(nameval[1]!="0");
            if (nameval[0]=="vars")
                groupAttributes[index].vars=nameval[1].split(",",QString::SkipEmptyParts);
        }
    }
}

void X2GoGroupAdminWindow::initGroupList(const QString& res)
{


    if (!groupAttributes[GID].supported)
    {
        treeGroups->hideColumn(COL_GID);
        lGid->hide();
        sbGid->hide();
    }
    if (!groupAttributes[DESC].supported)
    {
        treeGroups->hideColumn(COL_DESC);
        lDesc->hide();
        eDesc->hide();
    }
    if (!groupAttributes[PASSWORD].supported)
    {
        pbPass->hide();
    }

    if (!groupAttributes[TYPE].supported)
    {
        lType->hide();
        cbType->hide();
        treeGroups->hideColumn(COL_TYPE);
    }

    treeGroups->header()->setResizeMode ( QHeaderView::ResizeToContents );
    treeAllUsers->header()->setResizeMode ( QHeaderView::ResizeToContents );

    treeGroups->header()->setSortIndicator ( 0,Qt::AscendingOrder );
    treeAllUsers->header()->setSortIndicator ( 1,Qt::AscendingOrder );


    int firstUserPos=res.indexOf("BEGIN_USERS\n");
    int lastUserPos=res.indexOf("END_USERS");
    QString users=res.mid(firstUserPos,lastUserPos-firstUserPos);
    users.replace("BEGIN_USERS\n","");


    int firstGroupPos=res.indexOf("BEGIN_GROUPS\n");
    int lastGroupPos=res.indexOf("END_GROUPS");
    QString groups=res.mid(firstGroupPos,lastGroupPos-firstGroupPos);
    groups.replace("BEGIN_GROUPS\n","");

    treeAllUsers->setRootIsDecorated(true);
    treeAllUsers->setItemsExpandable(true);
    QStringList ulist=users.split("\n",QString::SkipEmptyParts);
    QString uentry;
    QList<struct user> listOfUsers;
    foreach(uentry, ulist)
    {
        QStringList udetails=uentry.split(":",QString::SkipEmptyParts);
        QString detail;
        struct user us;
        foreach(detail,udetails)
        {
            QStringList fields=detail.split("=");
            QString name=fields[0];
            QString value=fields[1];
            if (name == "UID")
                us.login=value;
            if (name == "FNAME")
                us.fname=value;
            if (name == "LNAME")
                us.lname=value;
            if (name == "UIDNUMBER")
                us.uidn=value;
        }
        listOfUsers<<us;
    }

    QStringList glist=groups.split("\n",QString::SkipEmptyParts);
    QString gentry;
    foreach(gentry, glist)
    {
        QStringList members;
        QTreeWidgetItem* it=new QTreeWidgetItem ( treeAllUsers );
        it->setSizeHint(0,QSize(39,39));
        it->setIcon(0,QPixmap ( ":/png/group.png" ));

        QTreeWidgetItem* grit=new QTreeWidgetItem(treeGroups);
        grit->setSizeHint(0,QSize(39,39));


        QStringList gdetails=gentry.split(":",QString::SkipEmptyParts);
        QString gname;
        QStringList memberUids;
        QString detail;
        foreach(detail,gdetails)
        {
            QStringList fields=detail.split("=");
            QString name=fields[0];
            QString value=fields[1];
            if (name == "NAME")
            {
                gname=value;
                grit->setText(COL_GNAME,value);
            }
            if (name == "MEMBERS")
                memberUids=value.split(";");
            if (name=="TYPE")
                grit->setText(COL_TYPE,value);
            if (name=="GID")
                grit->setText(COL_GID,value);
            if (name=="DESC")
                grit->setText(COL_DESC,value);
        }
//         it->setFlags(it->flags()&~Qt::ItemIsSelectable);
        it->setChildIndicatorPolicy(QTreeWidgetItem::DontShowIndicatorWhenChildless);
        QString memberUid;
        int numOfItems=0;
        foreach(memberUid, memberUids)
        {
            user us;
            us.login=memberUid;
            int ind=listOfUsers.indexOf(us);
            if (ind!=-1)
            {
                ++numOfItems;
                QTreeWidgetItem* chit=new QTreeWidgetItem ( it);
                chit->setSizeHint(0,QSize(39,39));

                chit->setIcon(0,QPixmap ( ":/png/user.png" ));
                chit->setText(1,listOfUsers[ind].login);
                members<<listOfUsers[ind].login;
                chit->setChildIndicatorPolicy(QTreeWidgetItem::DontShowIndicator);
            }
        }
        it->setText(1,gname+"("+QString::number(numOfItems)+")");
        grit->setData(COL_GNAME,Qt::UserRole,members);
    }
    gbAllGroups->setEnabled(true);

}


bool X2GoGroupAdminWindow::stringIsAscii(const QString& str)
{
    QByteArray ba=str.toAscii();
    for (int i=0; i<ba.size();++i)
        if (!isascii(ba[i]))
            return false;
    return true;
}


int X2GoGroupAdminWindow::applyQestion()
{
    return QMessageBox::question(this,tr("Apply changes"),tr("Do you want to apply changes?"),
                                 QMessageBox::Yes, QMessageBox::No, QMessageBox::Cancel);
}


bool X2GoGroupAdminWindow::closePlugin()
{
    if (closing)
        return false;
    if (changed)
    {
        int res=applyQestion();
        if (res==QMessageBox::Cancel)
            return false;
        if (res==QMessageBox::Yes)
        {
            closing=true;
            slotApply();
            return false;
        }
    }
    hidden=true;
    return true;
}


void X2GoGroupAdminWindow::setCenterInstance(CenterInterface* obj)
{
    centerInstance=obj;
}


void X2GoGroupAdminWindow::slotHandleSslErrors(QNetworkReply* reply, const QList< QSslError >& errors)
{
    if (hidden)
        return;
    centerInstance->handleSslErrors(reply, errors);
}


void X2GoGroupAdminWindow::slotRpcFault(int, const QString& errMessage)
{
    if (hidden)
        return;
    QMessageBox::critical(this, tr("Connection error"), errMessage);
    centerInstance->backToList();
}


bool X2GoGroupAdminWindow::checkResult(const QString& result)
{
    if (result.indexOf("SERVERDOWN")==0)
    {
        QString res=result;
        res.replace("SERVERDOWN ","" );
        QString errorMessage=tr("Server connection failed<br>Error message:")+"\n"+res;
        QMessageBox::critical(this, tr("Server error"), errorMessage);
        return false;
    }
    if (result.indexOf("NOACCESS")==0)
    {
        QString res=result;
        res.replace("NOACCESS ","" );
        QString errorMessage=tr("Login failed");
        QMessageBox::critical(this, tr("Server error"), errorMessage);
        return false;
    }
    if (result.indexOf("CMDERR")==0)
    {
        QString res=result;
        res.replace("CMDERR ","" );
        QString errorMessage=tr("Execution of command failed<br>Error message:")+"\n"+res;
        QMessageBox::critical(this, tr("Server error"), errorMessage);
        return false;
    }
    return true;
}

void X2GoGroupAdminWindow::slotNewGroup()
{
    currentGroup.newGroup=true;
    currentGroup.gid=QString::null;
    currentGroup.desc=QString::null;
    slotGroupSelected(0,0);
}


void X2GoGroupAdminWindow::slotGroupSelected(QTreeWidgetItem* item, int col)
{
    if (col ==-1)
        return;
    if (changed)
    {
        int res=applyQestion();
        if (res==QMessageBox::Cancel)
            return;
        if (res==QMessageBox::Yes)
        {
            slotApply();
            return;
        }
    }
    cbType->clear();
    listGroupUsers->clear();
    gbGroup->setEnabled(true);
    if (item)
    {
        lName->setText(item->text(COL_GNAME));
        lName->show();
        lNameLabel->hide();
        eName->hide();
        currentGroup.users=item->data(COL_GNAME,Qt::UserRole).toStringList();
        currentGroup.users.sort();
        listGroupUsers->addItems(currentGroup.users);

        currentGroup.newGroup=false;
        //add support for other atributes
        currentGroup.password=currentGroup.newpassword=QString::null;
        currentGroup.item=item;
        if (groupAttributes[GNAME].supported)
        {
            lName->setText(item->text(COL_GNAME));
            currentGroup.name=item->text(COL_GNAME);
        }
        if (groupAttributes[TYPE].supported)
        {
            cbType->insertItem(0,item->text(COL_TYPE));
            cbType->setCurrentIndex(0);
            if (!groupAttributes[TYPE].editable)
                cbType->setEnabled(false);
        }

        if (groupAttributes[DESC].supported)
        {
            eDesc->setText(item->text(COL_DESC));
            currentGroup.desc=item->text(COL_DESC);
        }
        sbGid->setValue(item->text(COL_GID).toInt());
        currentGroup.gid=item->text(COL_GID);

        eDesc->setVisible(groupAttributes[DESC].supported);
        eDesc->setEnabled(groupAttributes[DESC].editable);
        pbPass->setEnabled(groupAttributes[PASSWORD].editable);
        listGroupUsers->setEnabled(groupAttributes[MEMBERS].editable);
        pbAddUsers->setEnabled(groupAttributes[MEMBERS].editable);
        treeAllUsers->setEnabled(groupAttributes[MEMBERS].editable);
        pbRemoveUsers->setEnabled(groupAttributes[MEMBERS].editable);
        pbApply->setEnabled(false);
        changed=false;
    }
    else
    {
        //add support for other attributes
        lName->hide();
        lNameLabel->show();
        eName->clear();
        eName->show();
        eDesc->clear();
        if (groupAttributes[TYPE].supported)
        {
            cbType->insertItems(0,groupAttributes[TYPE].vars);
            cbType->setCurrentIndex(0);
            cbType->setEnabled(true);
        }
        if(groupAttributes[GID].supported)
	{
	  sbGid->setValue(findFreeGid());
	}

        qDebug()<<groupAttributes[PASSWORD].oncreate<<groupAttributes[MEMBERS].oncreate;
        pbPass->setEnabled(groupAttributes[PASSWORD].editable&&groupAttributes[PASSWORD].oncreate);
        listGroupUsers->setEnabled(groupAttributes[MEMBERS].editable&&groupAttributes[MEMBERS].oncreate);
        pbAddUsers->setEnabled(groupAttributes[MEMBERS].editable&&groupAttributes[MEMBERS].oncreate);
        treeAllUsers->setEnabled(groupAttributes[MEMBERS].editable&&groupAttributes[MEMBERS].oncreate);
        pbRemoveUsers->setEnabled(groupAttributes[MEMBERS].editable&&groupAttributes[MEMBERS].oncreate);
        slotGroupChanged();
    }
    pbPass->setVisible(groupAttributes[PASSWORD].supported);
    listGroupUsers->setVisible(groupAttributes[MEMBERS].supported);
    pbAddUsers->setVisible(groupAttributes[MEMBERS].supported);
    treeAllUsers->setVisible(groupAttributes[MEMBERS].supported);
    pbRemoveUsers->setVisible(groupAttributes[MEMBERS].supported);
}

void X2GoGroupAdminWindow::slotRemoveUsers()
{
    for (int i=listGroupUsers->count()-1;i>=0;--i)
    {
        if (listGroupUsers->item(i)->isSelected())
        {
            delete listGroupUsers->takeItem(i);
            slotGroupChanged();
        }
    }
}

void X2GoGroupAdminWindow::slotAddUsers()
{
    QTreeWidgetItemIterator it(treeAllUsers);
    while (*it)
    {
        QTreeWidgetItem* item=(*it);
        if (item->parent()!=0)
        {
            if (item->isSelected()|| item->parent()->isSelected())
            {
                QList<QListWidgetItem*> lst=listGroupUsers->findItems(item->text(1),Qt::MatchExactly);
                if (lst.size()==0)
                {
                    listGroupUsers->addItem(item->text(1));
                    slotGroupChanged();
                }
            }
        }
        ++it;
    }
    listGroupUsers->sortItems();
}

void X2GoGroupAdminWindow::slotChangePass()
{
    bool ok;
    QString ps=QInputDialog::getText(this,tr("Set group password"), tr("Group password:"), QLineEdit::Password, "", &ok);
    if (ok)
    {
        QString ps1=QInputDialog::getText(this,tr("Verify group password"), tr("Verify password:"), QLineEdit::Password, "", &ok);
        if (ps != ps1)
        {
            QMessageBox::critical(this,tr("Error"),tr("Passwords are not identical"));
            return;
        }
        if (ps1.length()<=0)
        {
            QMessageBox::critical(this,tr("Error"),tr("Password required"));
            return;
        }
        currentGroup.newpassword=ps;
        slotGroupChanged();
    }
}


void X2GoGroupAdminWindow::slotGroupChanged()
{
    changed=true;
    pbApply->setEnabled(true);
}

void X2GoGroupAdminWindow::slotDeleteGroups()
{
    if (changed)
    {
        int res=applyQestion();
        if (res==QMessageBox::Cancel)
            return;
        if (res==QMessageBox::Yes)
        {
            slotApply();
            return;
        }
    }
    QStringList groups;
    QTreeWidgetItemIterator it(treeGroups);
    while (*it)
    {
        if ((*it)->isSelected())
            groups<<(*it)->text(COL_GNAME);
        ++it;
    }
    if (!groups.count())
        return;

    if (QMessageBox::warning(this,tr("Warning"),tr("Delete selected group(s)"),QMessageBox::Yes, QMessageBox::No)!=QMessageBox::Yes)
        return;

    QVariantList args;
    args<<centerInstance->getLogin()<<centerInstance->getPassword()<<groups.join(";");

    rpc->call("x2goadmin.removeGroups",args, this, SLOT(slotGroupsRemoved(QVariant&)),
              this, SLOT(slotRpcFault(int, const QString&)));

    gbGroup->setEnabled(false);
    gbAllGroups->setEnabled(false);
    centerInstance->setStatus(tr("Retrieving information from server, please wait..."));
}

void X2GoGroupAdminWindow::slotGroupsRemoved(QVariant& result)
{
    if (hidden)
        return;
    centerInstance->setStatus(tr("Ready"));
    QString res=result.toString();
    if (!checkResult(res))
    {
        centerInstance->backToList();
        return;
    }
    startPlugin();

}

bool X2GoGroupAdminWindow::isAttributeUnic(int col, QString value)
{
    QTreeWidgetItemIterator it(treeGroups);
    while (*it)
    {
        if ((*it)->text(col)==value)
            return false;
        ++it;
    }
    return true;
}

int X2GoGroupAdminWindow::findFreeGid()
{
    QVector<int> gids;
    QTreeWidgetItemIterator it(treeGroups);
    while (*it)
    {
        gids<<(*it)->text(COL_GID).toInt();
        ++it;
    }
    qSort(gids);
    int gid=1000;
    foreach (int id, gids)
    {
        if (id > gid)
            return gid;
        if (id==gid)
            ++gid;
    }
    return gid;
}


Q_EXPORT_PLUGIN2(x2gogroupadmin, X2GoGroupAdminWindow)
