/*
    Part of X2GoAdminCenter.
    Copyright (C) 2011-2015 Oleksandr Shneyder<oleksandr.shneyder@obviously-nice.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef X2GOGROUPADMINWINDOW_H
#define X2GOGROUPADMINWINDOW_H

#include <x2goadminInterface.h>

#include "ui_x2gogroupadminui.h"
#include <maiaXmlRpcClient.h>
#include <QLocale>

#define VERSION "0.0.0.1"


#define APPICON QPixmap(":/png/x2gogroup.png")
#define GROUPATTR TYPE+1


struct user {
    QString login;
    QString fname;
    QString lname;
    QString uidn;
    bool operator == (const struct user& us)
    {
        return (us.login==login);
    }
};

class X2GoGroupAdminWindow :public QWidget, public X2goAdminPlugin, private Ui::X2GoGroupAdminUI
{
    Q_OBJECT
    Q_INTERFACES(X2goAdminPlugin)
public:
    X2GoGroupAdminWindow();
    ~X2GoGroupAdminWindow();
    QString pluginName() {
        return NAME;
    }
    QString description() {
        return DESCRIPTION;
    }
    QString about() {
        return ABOUT;
    }
    QPixmap pluginPic() {
        return APPICON;
    }
    QWidget* widget() {
        return this;
    }
    bool closePlugin();
    void setCenterInstance(CenterInterface *obj);
    void startPlugin();

private:
    void getGroups();
    bool checkResult(const QString& result);
    int applyQestion();
    bool stringIsAscii(const QString& str);
    void initGroupAttr(const QString& attrString);
    void initGroupList(const QString& res);
    void installTranslator();
    bool isAttributeUnic(int col, QString value);
    int findFreeGid();
    CenterInterface* centerInstance;
    MaiaXmlRpcClient* rpc;
    bool changed;
    bool hidden;
    bool closing;
    bool adminMode;
    enum {GNAME, GID, DESC, PASSWORD, MEMBERS, TYPE};
    enum {COL_GNAME, COL_GID, COL_DESC, COL_TYPE};
    struct grattr
    {
        bool supported;
        bool editable;
        bool unicode;
        bool oncreate;
        QStringList vars;
    };
    grattr groupAttributes[GROUPATTR];
    struct {
        QString name,gid,desc,type,password,newpassword;
        QStringList users;
        QTreeWidgetItem *item;
        bool newGroup;
    }currentGroup;


    QString NAME;
    QString DESCRIPTION;
    QString DEVELOPERS;
    QString TRANSLATORS;
    QString ABOUT;


private slots:
    void slotHandleSslErrors(QNetworkReply *reply, const QList<QSslError> &errors);
    void slotRpcFault(int, const QString& errMessage);
    void slotGroupList(QVariant& var);
    void slotApply();
    void slotGroupSelected(QTreeWidgetItem* item,int col);
    void slotRemoveUsers();
    void slotAddUsers();
    void slotGroupChanged();
    void slotGroupModified(QVariant& var);
    void slotChangePass();
    void slotNewGroup();
    void slotDeleteGroups();
    void slotGroupsRemoved(QVariant& result);
};

#endif // X2GOGROUPADMINWINDOW_H
