/*
    Part of X2GoAdminCenter.
    Copyright (C) 2011-2015 Oleksandr Shneyder<oleksandr.shneyder@obviously-nice.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef X2GOSERVERADMINWINDOW_H
#define X2GOSERVERADMINWINDOW_H

#include <x2goadminInterface.h>

#include "ui_x2gosessionadminui.h"
#include <maiaXmlRpcClient.h>
#include <QLocale>

#define VERSION "0.0.0.1"

#define APPICON QPixmap(":/png/x2gosession.png")

struct NodeOrServer
{
    QString name,ip,desc;
    bool status;
    QStringList nodes;
    bool operator == (const struct NodeOrServer& s)
    {
        return (s.name==name);
    }
};

class X2GoSessionAdminWindow :public QWidget, public X2goAdminPlugin, private Ui::X2GoSessionAdminUI
{
    Q_OBJECT
    Q_INTERFACES(X2goAdminPlugin)
public:
    X2GoSessionAdminWindow();
    ~X2GoSessionAdminWindow();
    QString pluginName() {
        return NAME;
    }
    QString description() {
        return DESCRIPTION;
    }
    QString about() {
        return ABOUT;
    }
    QPixmap pluginPic() {
        return APPICON;
    }
    QWidget* widget() {
        return this;
    }
    bool closePlugin();
    void setCenterInstance(CenterInterface *obj);
    void startPlugin();

private:
    bool checkResult(const QString& result);
    void installTranslator();
    CenterInterface* centerInstance;
    MaiaXmlRpcClient* rpc;
    bool changed;
    bool hidden;
    bool closing;
    bool adminMode;
    QString selectedSession;


    enum {COL_USER, COL_STATE, COL_SERVER, COL_START, COL_DISPLAY, COL_IP, COL_INTERRUPTED, COL_ID};

    QString NAME;
    QString DESCRIPTION;
    QString DEVELOPERS;
    QString TRANSLATORS;
    QString ABOUT;

    QTimer servTimer;


private slots:
    void slotGetSessions();
    void slotHandleSslErrors(QNetworkReply *reply, const QList<QSslError> &errors);
    void slotRpcFault(int, const QString& errMessage);
    void slotSessionsList(QVariant& var);
    void slotRetControl(QVariant& var);
    void slotSelectSession(QTreeWidgetItem* item, int);
    void slotSuspend();
    void slotTerminate();
};

#endif // X2GOGROUPADMINWINDOW_H
