/*
    Part of X2GoAdminCenter.
    Copyright (C) 2011-2015 Oleksandr Shneyder <oleksandr.shneyder@obviously-nice.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "x2goadmincenter.h"
#include "x2goadminInterface.h"
#include <QtGui>
#include "x2gosettings.h"
#include "settingsdialog.h"
#include <QInputDialog>


void X2GoAdminCenter::installTranslator()
{
    QTranslator* trans=new QTranslator();
    QString filename=QString ( ":/x2goadmincenter_%1" ).arg (
                         QLocale::system().name() );
    filename=filename.toLower();
    if ( !trans->load ( filename ) )
    {
        qDebug ( "Can't load translator (%s) !\n",
                 filename.toLocal8Bit().data() );
    }
    else
    {
        QCoreApplication::installTranslator ( trans );
    }


    QTranslator* qtTranslator=new QTranslator;
    filename=QString ( ":/qt_%1" ).arg ( QLocale::system().name() );
    if ( !qtTranslator->load ( filename ) )
    {
        qDebug()<< "Can't load translator "<<
        filename.toLocal8Bit().data() ;
    }
    else
    {
        QCoreApplication::installTranslator ( qtTranslator );
    }
}


X2GoAdminCenter::X2GoAdminCenter()
{
    installTranslator();

    NAME =tr("X2Go administration center");
    DESCRIPTION= tr("Administration plugins");
    DEVELOPERS= "Oleksandr Shneyder &lt;oleksandr.shneyder@obviously-nice.de&gt;<br>"
                "Heinz-Markus Graesing &lt;heinz-m.graesing@obviously-nice.de&gt;";

    TRANSLATORS ="Heinz-Markus Graesing &lt;heinz-m.graesing@obviously-nice.de&gt;";

    ABOUT= "<b>"+NAME + "-" + VERSION + "</b><br><br>"
           +DESCRIPTION+"<br><br><b>"
           + tr("Developers:")+"</b><br>" + DEVELOPERS+ "<br><br><b>"
           + tr("Translators:")+"</b><br>"+ TRANSLATORS;


    setupUi(this);
    setWindowIcon(APPICON);
    setWindowTitle(NAME);
    action_Back_to_modules_list->setIcon(QPixmap(":/png/go-previous.png"));
    action_Quit->setIcon(QPixmap(":/png/application-exit.png"));
    action_Settings->setIcon(QPixmap(":/png/configure.png"));
    actionSet_password->setIcon(QPixmap(":/png/pass.png"));

    actionAbout->setIcon(APPICON);
    actionAbout->setText(tr("About")+" "+NAME);

    action_Back_to_modules_list->setToolTip(tr("Back to overview"));
    action_Settings->setToolTip(tr("Settings ..."));
    actionSet_password->setToolTip(tr("Connection  password ..."));

    toolBar->addAction(action_Back_to_modules_list);
    tbSeparator= toolBar->addSeparator();
    toolBar->addAction(action_Settings);
    toolBar->addAction(actionSet_password);

    listWidget->setIconSize(QSize(128,128));

    action_Back_to_modules_list->setVisible(false);
    tbSeparator->setVisible(false);

    connect(action_Back_to_modules_list,SIGNAL(triggered(bool)), this, SLOT(slotBackToList()));
    connect(action_Quit,SIGNAL(triggered(bool)), this, SLOT(close()));

    connect(actionAbout_Qt, SIGNAL(triggered(bool)),qApp,SLOT(aboutQt()));
    connect(actionAbout,SIGNAL(triggered(bool)),this,SLOT(slotAbout()));
    connect(actionAbout_module,SIGNAL(triggered(bool)),this,SLOT(slotAboutModule()));
    connect(action_Settings,SIGNAL(triggered(bool)),this,SLOT(slotSettings()));
    connect(actionSet_password,SIGNAL(triggered(bool)),this,SLOT(slotSetPassword()));


    currentPlugin=0l;
    loadPlugins();
}

X2GoAdminCenter::~X2GoAdminCenter()
{

}

void X2GoAdminCenter::loadPlugins()
{
    QDir pluginsDir = QDir(qApp->applicationDirPath());
    pluginsDir.cd("../lib/x2goadmincenter/plugins");
    foreach (QString fileName, pluginsDir.entryList(QDir::Files))
    {
        QPluginLoader loader(pluginsDir.absoluteFilePath(fileName));
        QObject *plugin = loader.instance();
        if (plugin)
        {
            qDebug()<<fileName<<" plugin loaded";
            X2goAdminPlugin* iface=qobject_cast<X2goAdminPlugin *>(plugin);
            QListWidgetItem* it=new QListWidgetItem(listWidget);
            it->setIcon(iface->pluginPic());
            it->setText(iface->pluginName());
            it->setData(Qt::UserRole,(qulonglong)iface);
            it->setToolTip(iface->description());
            centralwidget->layout()->addWidget(iface->widget());
            iface->setCenterInstance(this);
            iface->widget()->hide();
        }
        else
            qDebug()<<fileName<<" loading failed!"<<loader.errorString();
    }
}

void X2GoAdminCenter::slotModuleClicked(QListWidgetItem* item)
{
    X2goAdminPlugin* plugin=(X2goAdminPlugin*)(item->data(Qt::UserRole).toULongLong());
    qDebug()<<"Selected plugin: "<<plugin->pluginName();
    listWidget->hide();
    action_Back_to_modules_list->setVisible(true);
    tbSeparator->setVisible(true);
    actionAbout_module->setVisible(true);
    actionAbout_module->setText(tr("About")+" "+plugin->pluginName());
    actionAbout_module->setIcon(plugin->pluginPic());
    setWindowIcon(plugin->pluginPic());
    setWindowTitle(tr("X2Go Administration Center")+" - "+plugin->pluginName());
    plugin->widget()->show();
    currentPlugin=plugin;
    statusBar()->setVisible(true);
    statusBar()->clearMessage();
    statusBar()->showMessage(plugin->pluginName());
    plugin->startPlugin();
}

void X2GoAdminCenter::slotBackToList()
{
    if (!currentPlugin)
        return;
    if (!currentPlugin->closePlugin())
        return;
    setWindowIcon(APPICON);
    action_Back_to_modules_list->setVisible(false);
    tbSeparator->setVisible(false);
    actionAbout_module->setVisible(false);
    setWindowTitle(NAME);
    currentPlugin->widget()->hide();
    listWidget->show();
    currentPlugin=0l;
    statusBar()->hide();
}

void X2GoAdminCenter::setStatus(const QString& message)
{
    statusBar()->showMessage(message);
}


void X2GoAdminCenter::closeEvent(QCloseEvent* event)
{
    if (currentPlugin)
        if (!currentPlugin->closePlugin())
        {
            event->ignore();
            return;
        }
    QMainWindow::closeEvent(event);
}

void X2GoAdminCenter::slotAbout()
{
    QMessageBox::about(this, NAME, ABOUT);
}

void X2GoAdminCenter::slotAboutModule()
{
    if (!currentPlugin)
        return;
    QMessageBox::about(this, currentPlugin->pluginName(), currentPlugin->about());
}


void X2GoAdminCenter::slotSettings()
{
    SettingsDialog dlg;
    dlg.exec();
}


QString X2GoAdminCenter::getHexVal(const QByteArray& ba)
{
    QStringList val;
    for ( int i=0;i<ba.size();++i )
    {
        QString bt;
        bt.sprintf ( "%02X", ( unsigned char ) ba[i] );
        val<<bt;
    }
    return val.join ( ":" );
}

void X2GoAdminCenter::handleSslErrors(QNetworkReply* reply, const QList< QSslError >& errors)
{
    QStringList err;
    QSslCertificate cert;
    for ( int i=0;i<errors.count();++i )
    {
        qDebug()<<"sslError ,code:"<<errors[i].error() <<":";
        err<<errors[i].errorString();
        if ( !errors[i].certificate().isNull() )
            cert=errors[i].certificate();
    }


    QString md5=getHexVal ( cert.digest() );
    QString fname=md5;
    fname=fname.replace(":","_");
    QUrl lurl (getUrl() );
    QString homeDir=QDir::homePath();
    if ( QFile::exists ( homeDir+"/.x2goadmincenter/ssl/exceptions/"+
                         lurl.host() +"/"+fname ) )
    {
        QFile fl ( homeDir+"/.x2goadmincenter/ssl/exceptions/"+
                   lurl.host() +"/"+fname );
        fl.open ( QIODevice::ReadOnly | QIODevice::Text );
        QSslCertificate mcert ( &fl );
        if ( mcert==cert )
        {
            reply->ignoreSslErrors();
            return;
        }
    }

    QString text=tr ( "<br><b>Invalid certificate sent by server</b><br><br>" );
    text+=err.join ( "<br>" );
    text+=tr ( "<p style='background:#FFFFDC;'>An exception should only be "
               "added if you are using a trusted internet connection. "
               "Be warned if you see this Message by connecting a known server.</p>");
    QMessageBox mb ( QMessageBox::Warning,tr ( "Secure connection failed" ),
                     text );
    text=QString::null;
    QTextStream ( &text ) <<err.join ( "\n" ) <<"\n"<<
    "------------\n"<<
    tr ( "Issued to:\n" ) <<
    tr ( "Common Name(CN)\t" ) <<
    cert.issuerInfo ( QSslCertificate::CommonName )
    <<endl<<
    tr ( "Organization(O)\t" ) <<
    cert.issuerInfo ( QSslCertificate::Organization )
    <<endl<<
    tr ( "Organizational Unit(OU)\t" ) <<
    cert.issuerInfo ( QSslCertificate::OrganizationalUnitName )
    <<endl<<
    tr ( "Serial Number\t" ) <<getHexVal ( cert.serialNumber() )
    <<endl<<endl<<
    tr ( "Issued by:\n" ) <<
    tr ( "Common Name(CN)\t" ) <<
    cert.subjectInfo ( QSslCertificate::CommonName )
    <<endl<<
    tr ( "Organization(O)\t" ) <<
    cert.subjectInfo ( QSslCertificate::Organization )
    <<endl<<
    tr ( "Organizational Unit(OU)\t" ) <<
    cert.subjectInfo ( QSslCertificate::OrganizationalUnitName )
    <<endl<<endl<<

    tr ( "Validity:\n" ) <<
    tr ( "Issued on\t" ) <<cert.effectiveDate().toString() <<endl<<
    tr ( "expires on\t" ) <<cert.expiryDate().toString() <<endl<<endl<<
    tr ( "Fingerprints:\n" ) <<
    tr ( "SHA1\t" ) <<
    getHexVal ( cert.digest ( QCryptographicHash::Sha1 ) ) <<endl<<
    tr ( "MD5\t" ) <<md5;



    mb.setDetailedText ( text );
    mb.setEscapeButton (
        ( QAbstractButton* ) mb.addButton ( tr ( "Abort connection" ),
                                            QMessageBox::RejectRole ) );
    QPushButton *okButton=mb.addButton ( tr ( "Add exception" ),
                                         QMessageBox::AcceptRole );
    mb.setDefaultButton ( okButton );

    mb.exec();
    if ( mb.clickedButton() == ( QAbstractButton* ) okButton )
    {
        qDebug()<<"accept certificate";
        QDir dr;
        dr.mkpath ( homeDir+"/.x2goadmincenter/ssl/exceptions/"+lurl.host() +"/" );
        QFile fl ( homeDir+"/.x2goadmincenter/ssl/exceptions/"+
                   lurl.host() +"/"+fname );
        fl.open ( QIODevice::WriteOnly | QIODevice::Text );
        QTextStream ( &fl ) <<cert.toPem();
        fl.close();
        reply->ignoreSslErrors();
    }
}




bool X2GoAdminCenter::adminMode()
{
    X2goSettings st("settings");
    return st.setting()->value("admin",true).toBool();
}

void X2GoAdminCenter::backToList()
{
    slotBackToList();
}

QString X2GoAdminCenter::getLogin()
{
    X2goSettings st("settings");
    return st.setting()->value("login","cipadmin").toString();
}

QSettings* X2GoAdminCenter::getSettings(QString group)
{
    X2goSettings* st=new X2goSettings(group);
    return st->setting();
}

QString X2GoAdminCenter::getPassword()
{
    if (password.length()<=0)
        slotSetPassword();
    return password;

}

void X2GoAdminCenter::slotSetPassword()
{
    bool ok;
    QString ps=QInputDialog::getText(this,tr("Connection password"), tr("Password:"), QLineEdit::Password, password,&ok);
    if (ok)
        password=ps;
}


QString X2GoAdminCenter::getUrl()
{
    X2goSettings st("settings");
    return st.setting()->value("url","http://www/cgi-bin/x2gorpcserver.cgi").toString();
}

